/*!\file drawFractal.h
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Fichier contenant les entêtes des fonctions présentes dans drawFractal.c correspondant au lancement de threads de dessin des différentes fractales
 *
 */

#include "ifs.h"
#include "flame.h"

/*!\fn void drawFractalM(SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, int* pi_nbIter, int* pi_numGradient)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Lance les threads de dessin de la fractale de Mandelbrot	
 *
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine la fractale de Mandelbrot
 * \param pd_zoom : Un pointeur vers un double correspondant au zoom
 * \param ppoint_pInit : Un pointeur vers un point correspondant au point initial de la fractale de Mandelbrot
 * \param pi_nbIter : Un pointeur vers un entier correspondant au nombre d'itérations
 * \param pi_numGradient : Un pointeur vers un entier correspondant à la couleur de la fractale de Mandelbrot
 *
 */

void drawFractalM(SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, int* pi_nbIter, int* pi_numGradient);


/*!\fn void drawFractalJ(SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, Point* ppoint_coeffC, int* pi_nbIter, int* pi_numGradient, int* pi_reverse)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Lance les threads de dessin de la fractale de Julia	
 *
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine la fractale de Julia
 * \param pd_zoom : Un pointeur vers un double correspondant au zoom
 * \param ppoint_pInit : Un pointeur vers un point correspondant au point initial de la fractale de Julia
 * \param ppoint_coeffC : Un pointeur vers un entier correspondant aux coefficients de Julia
 * \param pi_nbIter : Un pointeur vers un entier correspondant au nombre d'itérations
 * \param pi_numGradient : Un pointeur vers un entier correspondant à la couleur de la fractale de Mandelbrot
 * \param pi_reverse : Un pointeur vers un entier indiquant si la fractale de Julia doit être calculée par itération inverse ou non
 *
 */

void drawFractalJ(SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, Point* ppoint_coeffC, int* pi_nbIter, int* pi_numGradient, int* pi_reverse);


/*!\fn void drawFractalFlame(Flame* flame_flame)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Lance un thread de dessin de la fractale de type Flamme
 *
 * \pre 
 * \post 
 *
 * \param flame_flame : Structure correspondant à la fractale de type Flamme
 *
 */

void drawFractalFlame(Flame* flame_flame);


/*!\fn void drawFractalIfs(Ifs* ifs_ifs)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Lance un thread de dessin de la fractale de type Ifs
 *
 * \pre 
 * \post 
 *
 * \param ifs_ifs : Structure correspondant à la fractale de type Ifs
 *
 */

void drawFractalIfs(Ifs* ifs_ifs);


/*!\fn void runThreadMandelbrot(int i_numberThread, SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, int* pi_nbIter, int* pi_numGradient)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Lance un nombre de thread (passé en paramètre) de dessin de la fractale de Mandelbrot
 *
 * \pre 
 * \post 
 *
 * \param i_numberThread : Un entier correspondant au nombre de threads souhaitant être lancés
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine la fractale de Mandelbrot
 * \param pd_zoom : Un pointeur vers un zoom
 * \param ppoint_pInit : Un pointeur vers un point correspond au point initial de la fractale de Mandelbrot
 * \param pi_nbIter : Un pointeur vers un entier correspondant au nombre d'itérations 
 * \param pi_numGradient : Un pointeur vers un entier correspondant à la couleur de la fractale de Mandelbrot
 *
 */

void runThreadMandelbrot(int i_numberThread, SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, int* pi_nbIter, int* pi_numGradient);


/*!\fn void runThreadJulia(int i_numberThread, SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, Point* ppoint_coeffC, int* pi_nbIter, int* pi_numGradient, int* pi_reverse)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Lance un nombre de thread (passé en paramètre) de dessin de la fractale de Julia
 *
 * \pre 
 * \post 
 *
 * \param i_numberThread : Un entier correspondant au nombre de threads souhaitant être lancés
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine la fractale de Julia
 * \param pd_zoom : Un pointeur vers un zoom
 * \param ppoint_pInit : Un pointeur vers un point correspond au point initial de la fractale de Julia
 * \param ppoint_coeffC : Un pointeur vers un point correspond aux coefficients de Julia
 * \param pi_nbIter : Un pointeur vers un entier correspondant au nombre d'itérations
 * \param pi_numGradient : Un pointeur vers un entier correspondant à la couleur de la fractale de Mandelbrot
 * \param pi_reverse : Un pointeur vers un entier indiquant si la fractale de Julia doit être calculée par itération inverse ou non
 *
 */

void runThreadJulia(int i_numberThread, SDL_Surface* surf_screen, double* pd_zoom, Point* ppoint_pInit, Point* ppoint_coeffC, int* pi_nbIter, int* pi_numGradient, int* pi_reverse);
