/*!\file drawGeo.h
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Fichier contenant les entêtes des fonctions de dessin ou de forme géométrique
 *
 */


#include <SDL/SDL.h>

/*!\fn void drawPixel(SDL_Surface* surf_screen,  int i_x, int i_y, uint32_t uint_color)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Dessine un pixel
 *
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine le pixel
 * \param i_x : Un entier représentant l'abscisse d'un point
 * \param i_y : Un entier représentant l'ordonnée d'un point
 * \param uint_color : La couleur du pixel à dessiner
 *
 */

void drawPixel(SDL_Surface *surf_screen, int i_x, int i_y, uint32_t uint_color);


/*!\fn void drawLine(SDL_Surface* surf_screen, int i_x1,int i_y1, int i_x2,int i_y2);
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Dessine une ligne
 *
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine le pixel
 * \param i_x1 : Un entier représentant l'abscisse d'un point
 * \param i_y1 : Un entier représentant l'ordonnée d'un point
 * \param i_x2 : Un entier représentant l'abscisse d'un point
 * \param i_y2 : Un entier représentant l'ordonnée d'un point
 *
 */

void drawLine(SDL_Surface* surf_screen, int i_x1,int i_y1, int i_x2,int i_y2);


/*!\fn void drawLine2(SDL_Surface* surf_screen, int i_dx, int i_dy, int i_x1,int i_y1, int i_x2,int i_y2, int i_xincr, int i_yincr, uint32_t uint_color);
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Dessine une ligne
 *
 * \param surf_screen : Un pointeur vers un écran correspondant à l'écran sur lequel on dessine le pixel
 *
 * \param i_dx : Un entier représentant l'écart en abscisse
 * \param i_dy : Un entier représentant l'écart en ordonnée
 * \param i_x1 : Un entier représentant l'abscisse d'un point
 * \param i_y1 : Un entier représentant l'ordonnée d'un point
 * \param i_x2 : Un entier représentant l'abscisse d'un point
 * \param i_y2 : Un entier représentant l'ordonnée d'un point
 * \param i_xincr : Un entier représentant l'écart en abscisse
 * \param i_yincr : Un entier représentant l'écart en ordonnée
 * \param uint_color : Couleur de la ligne à dessiner
 *
 */

void drawLine2(SDL_Surface* surf_screen, int i_dx, int i_dy, int i_x1, int i_y1, int i_x2, int i_y2, int i_xincr, int i_yincr,  uint32_t uint_color);



/*!\fn double roundFunction(double d_num, double d_coeffRound)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief 
 *
 * \pre 
 * \post 
 *
 * \param d_num : Un réel à arrondir
 * \param d_coeffRound : Un réel représentant le coefficient d'arrondissement
 * \return Un réel arrondit 
 *
 */

double roundFunction(double d_num, double d_coeffRound);


