/*!\file point.h 
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Fichier d'entête des fonctions du fichier point.c
 *
 */


#ifndef _point_h_	
#define _point_h_

/*!\typedef Point
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type Point correspondant à la structure Point
 *
 */

typedef struct Point Point;


/*!\struct Point
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Structure désignant un point
 *
 */

struct Point{
double x; /*!<abscisse du point*/
double y; /*!< ordonnée du point*/
};

/*!\fn Point pointInit(double d_x, double d_y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Crée et initialise un point
 *
 * \pre 
 * \post 
 *
 * \param d_x : Un double correspondant à l'abscisse du point à initialiser
 * \param d_y : Un double correspondant à l'ordonnée du point à initialiser
 * \return Un point initalisé 
 *
 */

Point pointInit(double d_x, double d_y);


/*!\fn Point* pointInitAndSet(double d_x, double d_y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Crée, alloue, et initialise un pointeur vers un point
 *
 * \pre 
 * \post 
 *
 * \param d_x : Un double correspondant à l'abscisse du point à initialiser
 * \param d_y : Un double correspondant à l'ordonnée du point à initialiser
 * \return Un pointeur vers un point initalisé 
 *
 */

Point* pointInitAndSet(double d_x, double d_y);


/*!\fn Point* pointInitAndSetByPoint(Point point_point)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Crée, alloue et initialise un pointeur vers un point
 *
 * \pre 
 * \post 
 *
 * \param point_point : Un point correspond au point à partir duquel on initialise le nouveau point
 * \return Un pointeur vers un point initalisé 
 *
 */

Point* pointInitAndSetByPoint(Point point_point);


/*!\fn void pointSetValues(Point* ppoint_point, double d_x, double d_y)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Initialise un pointeur vers un point
 *
 * \pre 
 * \post 
 *
 * \param ppoint_point : Un pointeur vers un point correspondant au point à initialiser
 * \param d_x : Un double correspondant à l'abscisse du point à initialiser
 * \param d_y : Un double correspondant à l'ordonnée du point à initialiser
 *
 */

void pointSetValues(Point* ppoint_point, double d_x, double d_y);
#endif
