/*!\file stack.h
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Fichier contenant toutes les entêtes des fonctions relatives à la structure pile (stack)
 *
 */

#ifndef _stack_h_	
#define _stack_h_

/*!\typedef stack
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type stack correspondant à la structure stack
 *
 */

typedef struct stack stack;


/*!\typedef uCharDouble
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 12-01-2012 
 *
 * \brief Définition du type uCharDouble correspondant à la structure uCharDouble
 *
 */

typedef union uCharDouble uCharDouble;


/*!\union uCharDouble
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Union désigant soit un caractère, soit un double. Permet ainsi de manipuler (notamment avec les piles) des caractères comme des rééls
 *
 */


union uCharDouble {
	char c_value;/*!<un caractère*/
	double d_value;/*!<un réél*/
};


/*!\struct stack
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 09-01-2012 
 *
 * \brief Structure désignant une pile
 *
 */

struct stack {
 stack *prev;/*!<élément précédent*/
	uCharDouble ucd_value;/*!<valeur contenue dans une élément de la pile*/
};



/*!\fn uCharDouble stack_peek(stack *st_stack)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Retourne le sommet de la pile
 *
 * \pre 
 * \post 
 *
 * \param st_stack : Une pile correspondant à la pile dont on retourne le sommet
 * \return Un uCharDouble correspondant au sommet de la pile 
 *
 */

uCharDouble stack_peek(stack *st_stack);


/*!\fn void stack_push(stack** st_stack, uCharDouble ucd_value)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Empile une valeur
 *
 * \pre 
 * \post 
 *
 * \param st_stack : Un pointeur vers une pile depuis laquelle on empile
 * \param ucd_value : Une valeur correspondant à celle qu'on empile 
 *
 */

void stack_push(stack ** st_stack, uCharDouble ucd_value);



/*!\fn uCharDouble stack_pop(stack** st_stack)
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Dépile une valeur
 *
 * \pre 
 * \post 
 *
 * \param st_stack : Un pointeur vers une pile depuis laquelle on empile
 * \return Un uCharDouble correspondant à la valeur dépilée
 *
 */

uCharDouble stack_pop(stack** st_stack);

/*!\fn stack** stack_new();
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Crée un pointeur vers une pile et alloue la mémoire nécéssaire
 *
 * \pre 
 * \post 
 *
 * \return Un pointeur vers la pile crée
 *
 */

stack** stack_new();


/*!\fn void freeStack(stack** st_stack);
 *
 * \author SOUPLET Antoine
 * \version 1.0
 * \date 10-01-2012 
 *
 * \brief Libère l'espace mémoire alloué à un pointeur vers une pile
 *
 * \pre 
 * \post 
 *
 * \param st_stack Un pointeur vers la pile dont on libère la mémoire allouée
 *
 */


void freeStack(stack** st_stack);

#endif
